function gx = g(x)

% g provides the Dirichlet data for some homogeneous Laplace problem realized
% in examples/laplace/lshape1/*
%
% Usage: Y = g(X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/laplace/lshape1/*, we treat the Laplace problem
% 
%                -Laplace(u) = 0 in Omega with u = g on Gamma
%
%              We prescribe the exact solution in polar coordinates
%
%                u(r,theta) = r^{2/3} * cos(2theta/3)
%
%              which also be written in the complex plane by
%
%                u(a+bi) = Real( (a+bi)^(2/3) )
%
%              The evaluation points are given by the (N x 2)-matrix X, and the
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = g(X(j,:)).
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** obtain parameter alpha = 2/3
alpha = parameter();

%*** evaluate Dirichlet data
gx = real((x(:,1)+1i*x(:,2)).^alpha);

