function [alpha,beta,z] = parameters()

% parameters allows to modify the singularity in examples/laplace/lshape2/*
%
% Usage: [ALPHA,BETA,Z] = parameters()
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/lshape2/*, we treat the Laplace problem
% 
%                -Laplace(u) = 0 in Omega with u = g on Gamma
%
%              For given delta>0, we define a function in polar coordinates
%
%                v_delta(r,theta) = r^delta * cos(delta*theta)
%
%              We prescribe the exact solution of the Laplace equation by
%
%                u(x) = v_{2/3}(x) + v_{7/8}(x-z)
%
%              where z is the uppermost corner of Gamma. Then, v has a generic
%              singularity at the reentrant corner and an additional singularity
%              at the uppermost corner. Note that the function v_delta can be
%              written in the complex plane by
%
%                v_delta(a+bi) = Real( (a+bi)^delta )
%
%              which is used for the implementation. 
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** order of singularities
alpha = 2/3;
beta = 7/8;

%*** point for beta singularity
phi = 3*pi/4;
z = [1 -1] * [cos(phi) -sin(phi);sin(phi) cos(phi)]' /4;

