function gx = g(x)

% g provides the Dirichlet data for some homogeneous Laplace problem realized
% in examples/laplace/square/*
%
% Usage: Y = g(X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/laplace/square/*, we treat the Laplace problem
% 
%                -Laplace(u) = 0 in Omega with u = g on Gamma
%
%              We prescribe the (smooth) exact solution to be
%
%                u(x,y) = sinh(2pi*x)*cos(2pi*y)
%
%              The evaluation points are given by the (N x 2)-matrix X, and the
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = g(X(j,:)).
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

gx = sinh(2*pi*x(:,1)) .* cos(2*pi*x(:,2));

