function phix = phi(x,a,b)

% phi provides the Neumann data for some homogeneous Laplace problem realized
% in examples/laplace/square/*
%
% Usage: Y = phi(X,A,B)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/laplace/square/*, we treat the Laplace problem
% 
%                -Laplace(u) = 0 in Omega with u = g on Gamma
%
%              We prescribe the (smooth) exact solution to be
%
%                u(x,y) = sinh(2pi*x)*cos(2pi*y)
%
%              The evaluation points are given by the (N x 2)-matrix X. 
%              The (N x 2)-matrices A and B give the nodes of the corresponding
%              boundary elements, i.e., X(j,:) belongs to the line segment 
%              Ej = conv{ A(j,:) , B(j,:) }. This information is used to 
%              compute the outer unit normal vector on Ej. The            
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = phi(X(j,:)). 
%
% (C) 2009-2012 HILBERT-Team '09, '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** compute normal vector n on [a,b] for x in [a,b]
n = b-a;
n = [n(:,2) -n(:,1)] ./ repmat(sqrt(sum(n.^2,2)),1,2);

%*** compute gradient gradu(x)
grad(:,1) = 2*pi*cosh(2*pi*x(:,1)) .* cos(2*pi*x(:,2));
grad(:,2) = -2*pi*sinh(2*pi*x(:,1)) .* sin(2*pi*x(:,2));

%*** compute normal derivative d_n u(x) = gradu(x)*n(x)
phix = sum(grad.*n,2);

