addpath lib
addpath examples/paper/util

threads = [1, 2, 4, 8];

%*** Load mesh:
disp('Loading mesh from examples/poisson/lshape1 ...')
vertices = load( 'examples/poisson/lshape1/coordinates.dat' );
triangles = load( 'examples/poisson/lshape1/triangles.dat' );

%*** Refine the mesh uniform (4 times):
disp('Refining mesh a couple of times ...')
[vertices, triangles, coordinates, elements] = ...
	buildSortedMesh( vertices, triangles );
for i=1:4
	[vertices, triangles, elements] = ...
		refineMesh( vertices, triangles, elements );
end
[vertices, triangles, coordinates, elements] = ...
	buildSortedMesh( vertices, triangles );

%*** Refine the boundary mesh (4 times):
coordinates_fine = coordinates;
elements_fine = elements;
vertices_fine = vertices;
triangles_fine = triangles;
for i=1:4
	[vertices_fine, triangles_fine, elements_fine] = ...
		refineMesh( vertices_fine, triangles_fine, elements_fine );
end
[vertices_fine, triangles_fine, coordinates_fine, elements_fine] = ...
	buildSortedMesh( vertices_fine, triangles_fine );

%*** build K, V, W with 1, 2 and 4 threads for the boundary mesh and N with
%*** 1, 2 and 4 threads for the (coarse) volume mesh.
disp('Collecting data. This may take some time ...')
disp(['  Number of elements (for V, W, K): ' num2str(size(elements_fine, 1))])
disp(['  Number of triangles (for N): ' num2str(size(triangles, 1))])
data = threading( coordinates_fine, elements_fine, vertices, triangles, threads );

%*** Visualize results:
disp('Visualizing data ...');
loglog( threads, data(1, :), 'b.-', ...
	threads, data(2, :), 'ro-', ...
	threads, data(3, :), 'gx-', ...
	threads, data(4, :), 'c+-' );
legend( 'buildK', 'buildN', 'buildV', 'buildW' );
xlabel( 'Number of threads' );
ylabel( 'Time in seconds' );

