function [alpha,beta,z] = parameters()

% parameter allows to modify the singularity in examples/poisson/lshape2
%
% Usage: [ALPHA,BETA,Z] = parameter()
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/lshape2/, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega with u = g on Gamma (or GammaD)
%
%              We prescribe the exact solution by
%
%                u(x) = |x-z|^(2\beta) + r^{2/3} * cos(2theta/3)
%
%              where alpha = 2/3, beta = 0.9, and z = [0.14 0.14].
%
%              The exponents ALPHA, BETa and Z is provided by this function
%              so that
%              the user may easily change the order of the singularity.
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

alpha = 2/3;
beta = 0.9;
z = 0.14*[1 1];
