function phix = phi(x,a,b)

% phi provides the Neumann data for some Poisson problem realized 
% in examples/poisson/lshape3/
%
% Usage: Y = phi(X,A,B)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/lshape33, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega with du/dn = phi on Gamma (or GammaN)
%
%              We prescribe the exact solution by
%
%                u(x) = -1/4 (x1^2 + x2^2)
%
%              The evaluation points are given by the (N x 2)-matrix X. The
%              (N x 2)-matrices A and B give the nodes of the corresponding
%              boundary elements, i.e., X(j,:) belongs to the line segment 
%              Ej = conv{ A(j,:) , B(j,:) }. This information is used to 
%              compute the outer unit normal vector on Ej. The 
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = phi(X(j,:)). 
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

% polynomial part u 
% first order derivatives du/dx1 and du/dx2
d1u = -0.5*x(:,1);
d2u = -0.5*x(:,2);

%*** compute normal vector n on [a,b] for x in [a,b]
n = b-a;
n = [n(:,2) -n(:,1)] ./ repmat(sqrt(sum(n.^2,2)),1,2);

%*** evaluate normal derivative phi
phix = sum([d1u d2u].*n,2);
