function ux = g(x)

% g provides the Dirichlet data for some non-homogeneous Laplace problem realized
% in examples/poisson/lshape4/
%
% Usage: Y = g(X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/lshape4/, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega with u = g on Gamma (or GammaD)
%
%              We prescribe the exact solution by
%
%               u(r,theta) = -1/4(x^2 + y^2) + r^{2/3} * cos(2theta/3)
%
%              where (r,theta) are polar coordinates and (x,y) are
%              cartesian coordinates.
%
%              The evaluation points are given by the (N x 2)-matrix X, and the
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = g(X(j,:)).
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** exact solution is u(x) = ux(x) + u1(x), where
%*** u1(x,y) = Real( x+i*y)^alpha ) with alpha = 2/3
%*** and polynomial part u(x,y) = -1/4 (x^2 + y^2)

% polynomial part
ux = -0.25*(x(:,1).*x(:,1) + x(:,2).*x(:,2));

%*** "singular" part
alpha = parameters();

%*** evaluate u1(x)
u1x = real((x(:,1)+1i*x(:,2)).^alpha);

%*** obtain u(x)
ux =  ux + u1x;

