function fx = f(x)

% f provides the volume data for some Poisson problem realized 
% in examples/poisson/lshape5/
%
% Usage: Y = f(X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/lshape5, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega
%
%              We prescribe the exact solution by choosing u and f to be polynomials.
%              To be precise, we choose
%
%                u(x,y) = x(x-1/4)(y-1/4).
%
%              The evaluation points are given by the (N x 2)-matrix X. The
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = F(X(j,:)). 
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** exact solution is u(x,y) = x*(x-1/4)*(y-1/4)
fx = 0.5 - 2*x(:,2);
