function ux = g(x)

% g provides the Dirichlet data for some non-homogeneous Laplace problem realized
% in examples/poisson/lshape5/
%
% Usage: Y = g(X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/lshape5/, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega with u = g on Gamma (or GammaD)
%
%              We prescribe the exact solution by choosing u and f to be polynomials.
%              To be precise, we choose
%
%                u(x,y) = x(x-1/4)(y-1/4).
%
%              The evaluation points are given by the (N x 2)-matrix X, and the
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = g(X(j,:)).
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** exact solution is u(x,y) = x*(x-1/4)*(y-1/4)
ux = x(:,1).*(x(:,1)-0.25).*(x(:,2)-0.25);
