function fx = f(x)

% f provides the volume data for some Poisson problem realized 
% in examples/poisson/lshape6/
%
% Usage: Y = f(X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/lshape6, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega
%
%              We prescribe the exact solution in polar coordinates
%
%               u(r,theta) = x(x-1/4)(y-1/4) + r^{2/3} * cos(2theta/3)
%
%              where (r,theta) are polar coordinates and (x,y) are
%              cartesian coordinates. By choice of u it is seen that
%              f is a polynomial of degree 1.
%
%              The evaluation points are given by the (N x 2)-matrix X. The
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = F(X(j,:)). 
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** exact solution is u(x) = pi(x) + u1(x), where
%***   u1(x) = Real( x1+i*x2)^alpha );
%***   pi(x) = x1*(x1-1/4)*(x2-1/4)
fx = 0.5 - 2*x(:,2);
