function fx = f(x)

% f provides the volume data for some Poisson problem realized 
% in examples/poisson/zshape1/
%
% Usage: Y = f(X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              In examples/poisson/zshape1, we treat the Poisson problem
% 
%                -Laplace(u) = f in Omega
%
%              We prescribe the exact solution in polar coordinates
%
%                u(x) = |x-z|^(2*alpha).
%
%              where alpha = 0.51 and z = [0.1 0.1].
%
%              The evaluation points are given by the (N x 2)-matrix X. The
%              (N x 1)-vector Y provides the corresponding function values,
%              i.e., Y(j) = F(X(j,:)). 
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

%*** exact solution u(x) = |x-z|^(2*alpha) = [ (x1-z1)^2 + (x2-z2)^2 ]^alpha
[alpha,z] = parameters();

%*** u(x)
%u = sum(x.^2,2).^alpha;

%*** first-order derivatives du/dx1 and du/dx2
%d1u = 2*alpha*x(:,1).*sum(x.^2,2).^(alpha-1);
%d2u = 2*alpha*x(:,2).*sum(x.^2,2).^(alpha-1);

%*** second-order derivatives d2u/dx1 and d2u/dx2
x = x - repmat(z,length(x),1);
d11u = 2*alpha*sum(x.^2,2).^(alpha-1) ...
     + 4*alpha*(alpha-1)*x(:,1).^2.*sum(x.^2,2).^(alpha-2);
 
d22u = 2*alpha*sum(x.^2,2).^(alpha-1) ...
     + 4*alpha*(alpha-1)*x(:,2).^2.*sum(x.^2,2).^(alpha-2);

%*** f = -Laplace(u)
fx  = -(d11u + d22u);
