function N = buildN(varargin)

% buildN assembles the Galerkin matrix N corresponding to the newton-potential
% operator for P0(Omega)-ansatz and P0(Gamma)-test functions.
%
% Usage: N = buildN(COORDINATES,ELEMENTS,VERTICES,VOLUMES [,ETA])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%               
%              For a given partition {E1,...,EN} of a boundary piece Gamma
%              and a given partition {T1,...,TM} of a volume triangulation,
%              the newton-potential matrix N is an (N x M)-matrix with entries
%
%                Nkj = -1/(2 pi) \int_{Ek} \int_{Tj} 
%                                    log|x-y| dx ds(y)
%              
%              COORDINATES gives the coordinates for all nodes on Gamma.
%              ELEMENTS describes the elements on Gamma.
%              VERTICES gives the coordinates for all nodes in Omega.
%              VOLUMES describes the elements in Omega. If no optional
%              parameter ETA is given, a sane default value is chosen which is
%              defined by DEFAULT_ETA in geometry.h (0.5). For highly adapted
%              meshes, purely analytic computation leads do instabilities
%              due to cancellation effects. For ETA > 0, the outer integration
%              is replaced by Gaussian quadrature in case of
%
%              diam(Ek) <= ETA*dist(Ek,Tj).
%
% (C) 2010-2012 HILBERT-Team '10, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

error('Please run MAKE in HILBERT root directory to build integral operators');



