function Nf_x = evaluateN(varargin)

% evaluateN allows the pointwise evaluation (N*fh)(x) of the Newton
% potential N*fh  of a P0-function fh at points x.
%
% Usage: NF_X = evaluateN(VERTICES,VOLUMES,FH,X)
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Let {T1,...,TN} be a partition of the domain Omega.
%              VERTICES gives the coordinates for all nodes of the partition,
%              VOLUMES specifies the volume elements. The (N x 1)-vector FH
%              contains the elementwise values of the piecewise constant
%              density fh, i.e., fh|_{Tj} = FH(j). Let X be an
%              (M x 2)-matrix, where each row X(k,:) corresponds to some
%              point x, where (N*fh)(x) should be evaluated.
%              Then, the function evaluateN returns an (M x 1)-vector
%              NF_X with NF_X(j) = N*fh(X(j,:)).
%
% (C) 2011-2012 HILBERT-Team '11
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

error('Please run MAKE in HILBERT root directory to build integral operators');

% Author: Thomas Fuehrer
%
% History: TF  (19.10.2011 -> v. 0.4)
%          MK (23.11.2011 -> code review, leichte änderungen
