function Vphi_x = evaluateV(varargin)

% evaluateV allows the pointwise evaluation V*phih(x) of the simple-layer
% potential V*phih of a P0-function phih at points x.
%
% Usage:       VPHI_X = evaluateV(COORDINATES,ELEMENTS,PHIH,X [,ETA])
%
% Description: This file is part of the HILBERT program package for the
%              numerical solution of the Laplace equation with mixed
%              boundary conditions by use of BEM in 2D.
%
%              Let {E1,...,EN} be a partition of a boundary piece Gamma.
%              COORDINATES gives the coordinates for all nodes of the partition,
%              ELEMENTS specifies the boundary elements. The (N x 1)-vector PHIH
%              contains the elementwise values of the piecewise constant
%              density phih, i.e., phih|_{Ej} = PHIH(j). Let X be an
%              (M x 2)-matrix, where each row X(j,:) corresponds to some
%              evaluation point of V*phih(x) within the domain Omega or its
%              boundary Gamma. Then, the function evaluateV returns an
%              (M x 1)-vector VPHI_X with VPHI_X(j) = V*phih(X(j,:)).
%
%              ETA is an optional parameter that defines whether an evaluation
%              point XJ = X(J,:) and an element EK = ELEMENTS(K,:) are
%              admissable, i.e. whether there holds
%                  diam(EK) < ETA*dist(XJ, EK),
%              where diam(EK) denotes the diameter of the element EK and
%              dist(XJ,EK) denotes the distance between XJ and EK.
%
%              For each evaluation point, the integral is written as a sum of
%              element-wise integrals. If and evaluation point and an element
%              are not admissable, we compute the integral by using analytic
%              formulars and otherwise, we use Gaussian quadrature. This leads
%              to increased numerical stability.
%
%              In case that ETA is omitted, a sane default value is choosen.
%              In case that ETA=0, all integrals are evaluated analytically.
%
% (C) 2009-2012 HILBERT-Team '09, '12
% license + details:     http://www.asc.tuwien.ac.at/abem/hilbert
% support + bug report:  hilbert@asc.tuwien.ac.at
%
% Version: 3.0

error('Please run MAKE in HILBERT root directory to build integral operators');

% Author: Markus Mayr
%
% History: MM  (October 2009 -> v1.0)
%          DPR (11.11.2009 -> Matlab help)
%          DPR (12.11.2009 -> ok)
%          DPR (12.11.2009 -> error message if operators have not been built)

