function [diff, n1f, approxOfN1] = compareN1andApproxOfN1(coordinates,elements,...
    vertices,volumes,u,dudn,f)

addpath('../../lib/')

a = coordinates(elements(:,1),:);
b = coordinates(elements(:,2),:);

%*** evaluation points
x = 0.5*(a+b);
normalX = b-a;
normalX = [normalX(:,2) -normalX(:,1)]./repmat(sqrt(sum(normalX.^2,2)),1,2);
%*** Point to element vector
p2e = (1:size(elements,1))';

% nodal interpolation of u
uh = u(coordinates);

%***Gauss quadrature on [-1,1]
q_points_1d=[-1,1]/sqrt(3);
q_weights_1d=[1;1];
nQ_1d=2;

nE = size(elements,1);

%*** build vector of all quadrature nodes as (nQ*nE x 2)-matrix
a = coordinates(elements(:,1),:);
b = coordinates(elements(:,2),:);
sx = reshape(a,2*nE,1)*(1-q_points_1d) ...
        + reshape(b,2*nE,1)*(1+q_points_1d);
sx = 0.5*reshape(sx',nQ_1d*nE,2);

%*** sx2element(j) returns the element number k such that sx(j,:) lies on Ek
sx2element = reshape(repmat((1:nE),nQ_1d,1),nQ_1d*nE,1);

%*** Compute L^2-Proj of du/dn to P^0
dudnh = dudn(sx,a(sx2element,:),b(sx2element,:));
dudnh = 0.5*reshape(dudnh,nQ_1d,nE)'*q_weights_1d;

w_h = evaluateW(coordinates,elements,uh,x,normalX);
k_h = evaluateKadj(coordinates,elements,dudnh,x,normalX);

approxOfN1 = 0.5*dudnh - k_h - w_h;

%*** compute N1f with evaluateN1
tmp = (vertices(volumes(:,1),:)+vertices(volumes(:,2),:)...
    +vertices(volumes(:,3),:));
fh = f(3\tmp);
%*** Build Newton and Simple-layer, use standard eta
N = buildN(coordinates,elements,vertices,volumes);
V = buildV(coordinates,elements);
rhs = N*fh;
lambda_h = V\rhs;

n1f = evaluateN1(coordinates,elements,lambda_h,x,p2e);

diff = abs(n1f-approxOfN1);

end