function test1()
% test1.m
% Test the evaluation of (N1f)(x)
% by using the 2nd calderon identity [(0.5I-K')du/dn - Wu](x) = [N1f](x)
%
% Test1: Max/Mean/Min error of [(0.5I-K')du/dn - Wu](x) = [N1f](x) on square
% Test2: Max/Mean/Min error of [(0.5I-K')du/dn - Wu](x) = [N1f](x) on Lshape
%
% TF: 30.05.2012

addpath('../../lib/')

%*** Test 1
disp('## Test1')
disp('Omega = [0 0.25]^2, u = -0.25*(x^2+y^2) --> f=1')
%*** \Omega = [0 0.25]^2, u = -0.25*(x^2+y^2), f = 1
vertices = [0 0; 1 0; 1 1; 0 1; 0.5 0.5];
volumes = [1 2 5; 2 3 5; 3 4 5; 4 1 5];

coordinates = [0 0; 1 0; 1 1; 0 1];
elements = [1 2; 2 3; 3 4; 4 1];

vertices = 0.25*vertices;
coordinates = 0.25*coordinates;


nUnifRef = 6;
maxPerror = zeros(1,nUnifRef);
meanPerror = maxPerror;
minPerror = maxPerror;
nE = maxPerror;
nV = maxPerror;

for i=1:nUnifRef
    [coordinates elements] = refineBoundaryMesh(coordinates,elements);
    [vertices volumes] = refineVolumeMesh(vertices,volumes);
    nE(i) = size(elements,1);
    nV(i) = size(volumes,1);
    
    diff = abs(compareN1andApproxOfN1(coordinates,elements,...
    vertices,volumes,@u,@dudn,@f));
    maxPerror(i) = max(diff);
    meanPerror(i) = mean(diff);
    minPerror(i) = min(diff);
end


disp(sprintf('max(|[(1/2-Kadj)du/dn-Wu-N1f](xi)|) = %d, nE=%d, nV=%d\n',...
    reshape([maxPerror;nE;nV],3*nUnifRef,1)))
disp(sprintf('mean(|[(1/2-Kadj)du/dn-Wu-N1f](xi)|) = %d, nE=%d, nV=%d\n',...
    reshape([meanPerror;nE;nV],3*nUnifRef,1)))
disp(sprintf('min(|[(1/2-Kadj)du/dn-Wu-N1f](xi)|) = %d, nE=%d, nV=%d\n',...
    reshape([minPerror;nE;nV],3*nUnifRef,1)))

resultTest1 = [maxPerror; meanPerror; minPerror; nE; nV];


%*** Test 2
disp('## Test2')
disp('Omega = Lshape, u = -0.25*(x^2+y^2) --> f=1')
%*** \Omega = L-shape, u = -0.25*(x^2+y^2), f = 1
vertices = [0 0; 1 0; 1 1; 0 1; 0.5 0.5; ...
    -1 1; -1 0; -0.5 0.5; ...
    -1 -1; 0 -1; -0.5 -0.5];
volumes = [1 2 5; 2 3 5; 3 4 5; 4 1 5; ...
    1 4 8; 4 6 8; 6 7 8; 7 1 8; ...
    1 7 11; 7 9 11; 9 10 11; 10 1 11];

coordinates = [0 0; 1 0; 1 1; 0 1; -1 1; -1 0; -1 -1; 0 -1];
elements = [1 2; 2 3; 3 4; 4 5; 5 6; 6 7; 7 8; 8 1];

vertices = 0.25*vertices;
coordinates = 0.25*coordinates;

nUnifRef = 6;
maxPerror = zeros(nUnifRef,1);
meanPerror = maxPerror;
minPerror = maxPerror;
nE = maxPerror;
nV = maxPerror;

for i=1:nUnifRef
    [coordinates elements] = refineBoundaryMesh(coordinates,elements);
    [vertices volumes] = refineVolumeMesh(vertices,volumes);
    nE(i) = size(elements,1);
    nV(i) = size(volumes,1);
    
    diff = abs(compareN1andApproxOfN1(coordinates,elements,...
    vertices,volumes,@u,@dudn,@f));
    maxPerror(i) = max(diff);
    meanPerror(i) = mean(diff);
    minPerror(i) = min(diff);
end


disp(sprintf('max(|[(1/2-Kadj)du/dn-Wu-N1f](xi)|) = %d, nE=%d, nV=%d\n',...
    reshape([maxPerror;nE;nV],3*nUnifRef,1)))
disp(sprintf('mean(|[(1/2-Kadj)du/dn-Wu-N1f](xi)|) = %d, nE=%d, nV=%d\n',...
    reshape([meanPerror;nE;nV],3*nUnifRef,1)))
disp(sprintf('min(|[(1/2-Kadj)du/dn-Wu-N1f](xi)|) = %d, nE=%d, nV=%d\n',...
    reshape([minPerror;nE;nV],3*nUnifRef,1)))

resultTest2 = [maxPerror; meanPerror; minPerror; nE; nV];

%*** Save results to disc
%save resultsFromTEST1dotM resultTest1 resultTest2