function gx = g(x)

% g provides the Dirichlet data for some homogeneous Laplace problem realized in
%   demos/demo03/*
%
% Usage: Y = g(X)
%
% Descripition: This file is part of the ABEM2D-M program package for the
%               numerical solution of the Laplace equation with mixed
%               boundary conditions by use of BEM in 2D.
%
%               In example demos/demoSymm03/*, we treat the Laplace problem
% 
%                 -Laplace(u) = 0 in Omega with u = g on Gamma
%
%               We prescribe the (smooth) exact solution to be
%
%                 u(x,y) = sinh(2pi*x)*cos(2pi*y)
%
%               The evaluation points are given by the (N x 2)-matrix X, and the
%               (N x 1)-vector Y provides the corresponding function values,
%               i.e., Y(j) = g(X(j,:)).
%
% Author: Dirk Praetorius
%
% (C) M. Aurada, M. Ebner, S. Ferraz-Leite,
%     P. Goldenits, M. Mayr, D. Praetorius

% History: DPR (29.07.2009 --> v0.6)
% Version: 3.0           

gx = sinh(2*pi*x(:,1)) .* cos(2*pi*x(:,2));
