function phix = phi(x,a,b)

% phi provides the Neumann data for some homogeneous Laplace problem realized in
%     demos/demo03/*
%
% Usage: Y = PHI(X,A,B)
%
% Descripition: This file is part of the ABEM2D-M program package for the
%               numerical solution of the Laplace equation with mixed
%               boundary conditions by use of BEM in 2D.
%
%               In example demos/demoSymm03/*, we treat the Laplace problem
% 
%                 -Laplace(u) = 0 in Omega with u = g on Gamma
%
%               We prescribe the (smooth) exact solution to be
%
%                 u(x,y) = sinh(2pi*x)*cos(2pi*y)
%
%               The evaluation points are given by the (N x 2)-matrix X, and the
%               (N x 1)-vector Y provides the corresponding function values,
%               i.e., Y(j) = g(X(j,:)).
%
% Author: Dirk Praetorius
%
% (C) M. Aurada, M. Ebner, S. Ferraz-Leite,
%     P. Goldenits, M. Mayr, D. Praetorius

% History: DPR (29.07.2009 --> v0.6)
% Version: 3.0

%*** compute normal vector n on [a,b] for x in [a,b]
n = b-a;
n = [n(:,2) -n(:,1)] ./ repmat(sqrt(sum(n.^2,2)),1,2);

%*** compute gradient gradu(x)
grad(:,1) = 2*pi*cosh(2*pi*x(:,1)) .* cos(2*pi*x(:,2));
grad(:,2) = -2*pi*sinh(2*pi*x(:,1)) .* sin(2*pi*x(:,2));

%*** compute normal derivative d_n u(x) = gradu(x)*n(x)
phix = sum(grad.*n,2);