function phix = phi1(x,a,b)
% phi1 provides the Neumann data for some homogeneous Laplace problem
%
% Usage: Y = PHI(X,A,B)
%
% Descripition: This file is part of the HILBERT program package for the
%               numerical solution of the Laplace equation with mixed
%               boundary conditions by use of BEM in 2D.
%
%               We treat the Laplace problem
% 
%                 -Laplace(u) = 0 in Omega with u = g on Gamma
%
%               We prescribe the (smooth) exact solution to be
%
%                 u(x,y) = x
%
%               The evaluation points are given by the (N x 2)-matrix X, and the
%               (N x 1)-vector Y provides the corresponding function values,
%               i.e., Y(j) = g(X(j,:)).
%
% Author: Michael Karkulik
%
% (C) M. Aurada, M. Ebner, S. Ferraz-Leite,
%     P. Goldenits, M. Mayr, D. Praetorius

% History: DPR (28.11.2011 --> v0.6)
% Version: 3.0


%*** compute normal vector n on [a,b] for x in [a,b]
n = b-a;
n = [n(:,2) -n(:,1)] ./ repmat(sqrt(sum(n.^2,2)),1,2);

%*** compute the gradient
grad(:,1) = ones(size(x,1),1);
grad(:,2) = zeros(size(x,1),1);

phix = sum(grad.*n,2);
