clear all

addpath('../../../release1.0/lib/')
addpath('../../../release1.0/visualization/')
addpath('../')
addpath('../../lib/')

%******************************************************************************
%******************************************************* PROBLEM DEPENDEND DATA
%******************************************************************************

text = 'V*phi = (K+1/2)*g from example03';
filename = 'example03';

%*** use example01 for demonstration purposes
example_path = '../../../release1.0/examples/example03/';
addpath(example_path)
coordinates = load('coordinates.dat');
elements = load('elements.dat');
uD = @g;

%******************************************************************************
%******************************************************************************

%*** adaptivity parameter 
theta = 0.25;
counter = 1;
    
try
    %*** adaptive mesh-refining algorithm
    while 1
        
        %*** build operators V and K 
        V = buildV(coordinates,elements);
        K = buildK(coordinates,elements);
    
        %**************************************************** START: TEST K
        %*** testet K*1 = -1/2
        %******************************************************************
        h = sqrt(sum(( coordinates(elements(:,1),:) ...
                       - coordinates(elements(:,2),:) ).^2,2));
        testK = sum(K,2) + 1/2*h;
        %******************************************************************
    
        %************************************************** START: TEST VK1
        %*** testet V*phi = (K+1/2)*g fuer u(x,y) = x
        %******************************************************************
        a = coordinates(elements(:,1),:);
        b = coordinates(elements(:,2),:);
        midpoints = 0.5*(a + b);
        testVK1 = V*phi1(midpoints,a,b) ...
                  - ( 0.5*h.*g1(midpoints) + K*g1(coordinates) );
        %******************************************************************
    
        %************************************************** START: TEST VK2
        %*** testet V*phi = (K+1/2)*g fuer u(x,y) = y
        %******************************************************************
        testVK2 = V*phi2(midpoints,a,b) ...
                  - ( 0.5*h.*g2(midpoints) + K*g2(coordinates) );
        %******************************************************************
    
        %*** clear memory
        clear a b midpoints
        
        %*** build RHS for Symm's integral equation ( = buildSymmRHS)
        nE = size(elements,1);
        I = reshape(repmat(1:nE,2,1),2*nE,1);
        J = reshape(elements',2*nE,1);
        M = reshape(repmat(0.5*h,1,2)',2*nE,1);
        M = sparse(I,J,M);
        uDh = uD(coordinates);
        b = K*uDh + M*uDh*0.5;
        
        %*** clear memory
        clear K M
    
        %*** compute Galerkin solution
        x = V\b;
        
        %************************************************ START: TEST SOLVE
        %*** testet res = || b - A*x ||
        %******************************************************************
        testSolve = b - V*x;
        %******************************************************************
    
        %*** compute Galerkin solution by solving preconditioned system
        d = precondV(V,b);
        y = V\b;
        x = d.*y;
        
        %**************************************** START: TEST PRECOND-SOLVE
        %*** testet res = || b - A*x ||
        %******************************************************************
        testPrecSolve = b - V*y;
        %******************************************************************
    
        %*** compute weighted-residual error estimator rho
        rho_local = computeEstSlpResidual(coordinates,elements,x,uDh);
    
        %*** compute data oscillations
        osc_local = computeOscDirichlet(coordinates,elements,uD);
        
        %*** mark elements for refinement
        marked = markElements(theta,rho_local + osc_local);
    
        %*** plot estimators and test-norms
        dof(counter) = size(elements,1);
        rho(counter) = sqrt(sum(rho_local));
        osc(counter) = sqrt(sum(osc_local));
        testK_plot(counter) = norm(testK);
        testVK1_plot(counter) = norm(testVK1);
        testVK2_plot(counter) = norm(testVK2);
        testSolve_plot(counter) = norm(testSolve);
        testPrecSolve_plot(counter) = norm(testPrecSolve);
        figure(1)
        clf
        loglog(dof,rho,'r.-')
        hold on
        loglog(dof,osc,'b.-')
        loglog(dof,dof.^(-3/2),'k--')
        loglog(dof,testK_plot,'go-')
        loglog(dof,testVK1_plot,'ms-')
        loglog(dof,testVK2_plot,'cp-')
        loglog(dof,testSolve_plot,'k*-')
        loglog(dof,testPrecSolve_plot,'b<-')
        legend('rho','osc','order 3/2', ...
               'testK','testVK1','testVK2','testSolve','testPrecSolve',3)
        title(text)
        
        %*** save data
        tmp = int2str(counter);
        while length(tmp) < 3
            tmp = ['0',tmp];
        end
        eval(['save ',filename,'_',tmp,'.mat coordinates elements dof rho ',...
                'osc testK_plot testVK1_plot testVK2_plot testSolve_plot ',...
                'testPrecSolve_plot']);
        
        %*** generate new mesh
        [coordinates,elements] = refineBoundaryMesh(...
                                                coordinates,elements,marked);
    
        %*** iterate until instability occurs
        fprintf('number of elements: N = %d\r',size(elements,1))
        
        if size(elements,1) > 1000 && rho(counter) > 5*rho(counter-1)
           break
        end
       
    %    if size(elements,1) > 100
    %        break
    %    end
        
        counter = counter + 1;
    end
catch
    disp('Caught an exception in adaptive algorithm.');
    disp('Probably out of memory. Cleaning up ...');
end

rmpath(example_path)

eval(['print -depsc2 ',filename,'.eps'])
