function [ fails, passes, relative_errors ] = testDoubleSlp()

addpath('..');
  % In total, we run 150 tests.

  dblTol = 1e-8;
  fails = 0;
  passes = 0;
  relative_errors = zeros(150,1);

  for k = 0:4
    for l = 0:4
      for i = 1:2
        u = rand(2,1);
        v = rand(2,1);
        v(2) = (u(2)*v(1))/u(1);
        w = rand(2,1);

        f = @(s,t) f0(s,t,k,l,u,v,w);
        ref_sol = dblquad(f, -1,1, -1,1, dblTol)
        sol = mxDoubleSlp(k, l, u, v, w);
              
        if (abs(ref_sol - sol)/abs(ref_sol) < 1e-6)
          passes = passes + 1;
        else
          disp(sprintf('Test %d failed for k=%d,l=%d,parallel', ...
            10*k+2*l+i, k, l));
          disp(sprintf('  reference = %f, testing solution = %f', ...
            ref_sol, sol));
          disp(['  relative error = ' ...
                  num2str(abs(ref_sol - sol)/abs(ref_sol))]);
          disp(['  absolute error = ' num2str(abs(ref_sol-sol))]);
          fails = fails + 1;
        end
            
        relative_errors(10*(k)+2*(l)+i) = ...
            abs(ref_sol - sol)/abs(ref_sol);
      end
    end
  end
  
  for k = 0:4
    for l = 0:4
      for i = 1:2
        A = rand(2,2);
        while (det(A) < 1e-12)
          A = rand(2,2);
        end
        u = A(:,1);
        v = A(:,2);
        w = rand(2,1);
        
        while (all(w-u+v == [0; 0]) || all(w+u-v == [0; 0]))
          w = rand(2,1);
        end
              
        f = @(s,t) f0(s,t,k,l,u,v,w);
        ref_sol = dblquad(f, -1,1, -1,1, dblTol);
        sol = mxDoubleSlp(k, l, u, v, w);
              
        if (abs(ref_sol - sol)/abs(ref_sol) < 1e-6)
          passes = passes + 1;
        else
          disp(sprintf('Test %d failed for k=%d,l=%d,not parallel', ...
            50+10*k+2*l+i, k, l));
          disp(sprintf('  reference = %f, testing solution = %f', ...
            ref_sol, sol));
          disp(['  relative error = ' ...
                  num2str(abs(ref_sol - sol)/abs(ref_sol))]);
          disp(['  absolute error = ' num2str(abs(ref_sol-sol))]);
          fails = fails + 1;
	  % u and v can form a linear combination to yield w. compute the coefficients.
	  % if the coefficients are in [-1,1], then this indicates that dblquad has to deal with a singularity.
	  b = [u v]\w
        end
            
        relative_errors(50+10*k+2*l+i) = ...
            abs(ref_sol - sol)/abs(ref_sol);
      end
    end
  end
  
  for k = 0:4
    for l = 0:4
      A = rand(2,2);
      while (det(A) < 1e-12)
        A = rand(2,2);
      end
      u = A(:,1);
      v = A(:,2);
      w = v-u;
      
      f = @(s,t) f0(s,t,k,l,u,v,w);
      ref_sol = dblquad(f, -1,1, -1,1, dblTol);
      sol = mxDoubleSlp(k, l, u, v, w);
              
      if (abs(ref_sol - sol)/abs(ref_sol) < 1e-6)
        passes = passes + 1;
      else
        disp(sprintf('Test %d failed for k=%d,l=%d,not parallel', ...
          100+5*k+l, k, l));
        disp(sprintf('  reference = %f, testing solution = %f', ...
            ref_sol, sol));
        disp(['  relative error = ' ...
                num2str(abs(ref_sol - sol)/abs(ref_sol))]);
        disp(['  absolute error = ' num2str(abs(ref_sol-sol))]);
        fails = fails + 1;
      end
            
      relative_errors(100+5*k+l+1) = ...
          abs(ref_sol - sol)/abs(ref_sol);
    end
  end
  
  for k = 0:4
    for l = 0:4
      A = rand(2,2);
      while (det(A) < 1e-12)
        A = rand(2,2);
      end
      u = A(:,1);
      v = A(:,2);
      w = u-v;
      
      f = @(s,t) f0(s,t,k,l,u,v,w);
      ref_sol = dblquad(f, -1,1, -1,1, dblTol);
      sol = mxDoubleSlp(k, l, u, v, w);
              
      if (abs(ref_sol - sol)/abs(ref_sol) < 1e-6)
        passes = passes + 1;
      else
        disp(sprintf('Test %d failed for k=%d,l=%d,not parallel', ...
          125+5*k+l, k, l));
        disp(sprintf('  reference = %f, testing solution = %f', ...
            ref_sol, sol));
        disp(['  relative error = ' ...
                num2str(abs(ref_sol - sol)/abs(ref_sol))]);
        disp(['  absolute error = ' num2str(abs(ref_sol-sol))]);
        fails = fails + 1;
      end
            
      relative_errors(125+5*(k-1)+l) = ...
          abs(ref_sol - sol)/abs(ref_sol);
    end
  end  
end

function s = another_impl(k, l, u, v, w)
  mu = norm(v)/norm(u);
  s = mxSlp(0, u, w+v) + mxSlp(0, u, w-v) ...
      - mu*mxSlp(1, v, w+u) + mu*mxSlp(1, v, w-u);
  s = s / 2;
end

function val = f0(s, t, k, l, u, v, w)
  val = zeros(size(s));
  for i=1:length(s)
    val(i) = s(i)^k*t^l*log(norm(s(i)*u+t*v+w));
  end
end
