clear all
addpath('../lib/')
%addpath('../work/lib/')
%addpath('../examples/laplace/example01/')
addpath('../examples/laplace/lshape1/')
coordinates = load('coordinates.dat');
elements = load('elements.dat');
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]'/4;
uD = @g;

theta = 0.25;
counter = 1;

while 1
    
    [osc_,phih] = computeOscNeumann(coordinates,elements,@phi);
    
    %*** compute coarse-mesh Galerkin solution
    W = buildW(coordinates,elements) ...
             + buildHypsingStabilization(coordinates,elements);
    b = buildHypsingRHS(coordinates,elements,phih);
    x = W\b;
    
    %*** delete obsolete memory
    clear W b
    
    %*** compute residual error estimator and data oscillations
    
    uDh = uD(coordinates);
    ap=0.5*(coordinates(elements(:,1),:)+coordinates(elements(:,2),:));
    phiNh=phi(ap,coordinates(elements(:,1),:),coordinates(elements(:,2),:));
    rho_ = computeEstHypResidual(coordinates,elements,x,phiNh);
    %rhomod_ = computeEstHypResidualDirect(coordinates,elements,x,phiNh);
            
    %*** compute reliable error bound
    err_ = computeErrDirichlet(coordinates,elements,x,@g);
    
     
    %*** store computed data
    nE(counter) = size(elements,1);
    rho(counter) = sqrt(sum(rho_));
    %rhomod(counter) = sqrt(sum(rhomod_));
    osc(counter) = sqrt(sum(osc_));
    err(counter) = sqrt(sum(err_));
    
    %*** plot results
    figure(1)
    clf
    loglog(nE,err,'r.-')
    hold on
    loglog(nE,rho,'bx-')
    %loglog(nE,rhomod,'gx-')
    loglog(nE,osc,'m*-')
    loglog(nE,2\osc(1)/nE(1)^(-3/2)*nE.^(-3/2),'k--')
    loglog(nE,2*err(1)/nE(1)^(-3/2)*nE.^(-3/2),'k--')
    legend('err','rho','rho\_direct','osc','3/2',3)
    drawnow
    
%    %*** save meshes
%    if counter < 10
%        filename = ['testEstSlpResidual/','0',int2str(counter),'.mat'];
%    else
%        filename = ['testEstSlpResidual/',int2str(counter),'.mat'];
%    end
%    eval(['save ',filename,' coordinates elements']);
    
    %*** stop if computation becomes unstable
   if (nE(end) > 1000) 
       counter
       break
   end
   
    
    %*** otherwise, adaptive rho-based mesh-refinement
    marked = markElements(theta,1/4,rho_+osc_);
    %marked = markElements(theta,1/4,rhomod_+osc_);
    %marked = markElements(theta,rho_+osc_);
    %marked = markElements(theta,rhomod_+osc_);
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements,marked);
    
    counter = counter + 1;
end
hgsave('testEstHypsingResdiual.fig')