clear all

addpath('../work/lib/')
addpath('../work/helpers/')
addpath('../work/visualization/')
addpath('../work/examples/example04/')

coordinates = load('coordinates.dat');
elements = load('elements.dat');
nE = size(elements,1);
alpha = 3*pi/4;
coordinates = coordinates*[cos(alpha) -sin(alpha);sin(alpha) cos(alpha)]'/4;
uD = @g;

theta = 0.25;
counter = 1;

figure(1)
figure(2)
clf(1)
clf(2)

while 1

    %*** visualize Gamma and initial partition
%    figure(1)
%    for j = 1:nE
%        a = coordinates(elements(j,1),:);
%        b = coordinates(elements(j,2),:);
%        plot([a(1) b(1)],[a(2),b(2)],'o-k','MarkerFaceColor','r')
%        hold on
%    end
%    axis equal
    plotMesh(coordinates,elements,1);
 
    %*** compute coarse-mesh Galerkin solution
    V = buildV(coordinates,elements);
    b = buildSymmRHS(coordinates,elements,uD);
    x = V\b;
    
    %*** delete obsolete memory
    clear V b
    
    %*** compute residual error estimator and data oscillations
    osc_ = computeOscDirichlet(coordinates,elements,uD);
    uDh = uD(coordinates);
    rho_ = computeEstSlpResidual(coordinates,elements,x,uDh);

    %*** compute Faermann error estimator
    faermann_ = computeEstSlpFaermann(coordinates,elements,x,uDh);
    
    %*** compute reliable error bound
    err_ = computeErrNeumann(coordinates,elements,x,@phi);
    
    %*** compute fine-mesh Galerkin solution
    [coordinates_fine,elements_fine,father2son] ...
        = refineBoundaryMesh(coordinates,elements);
    V_fine = buildV(coordinates_fine,elements_fine);
    b_fine = buildSymmRHS(coordinates_fine,elements_fine,uD);
    x_fine = V_fine\b_fine;
   
    %*** compute eta-tilde and mu-tilde
    eta_tilde_ = computeEstSlpEtaTilde(father2son,V_fine,x_fine);
    mu_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,x_fine);
    
    %*** delete obsolete memory
    clear V_fine b_fine x_fine coordinates_fine elements_fine
    
    %*** store computed data
    nE(counter) = size(elements,1);
    rho(counter) = sqrt(sum(rho_));
    rho(counter) = sqrt(sum(rho_));
    osc(counter) = sqrt(sum(osc_));
    err(counter) = sqrt(sum(err_));
    eta(counter) = sqrt(eta_tilde_);
    mu(counter) = sqrt(sum(mu_tilde_));
    faermann(counter) = sqrt(sum(faermann_));
    %*** plot results
    figure(2)
    clf
    loglog(nE,err,'r.-')
    hold on
    loglog(nE,rho,'bx-')
    loglog(nE,eta,'cs-')
    loglog(nE,mu,'cd-')
    loglog(nE,osc,'m*-')
    loglog(nE,faermann,'g+-')
    loglog(nE,2\osc(1)/nE(1)^(-3/2)*nE.^(-3/2),'k--')
    loglog(nE,2*err(1)/nE(1)^(-3/2)*nE.^(-3/2),'k--')
    legend('err','rho','eta','mu','osc','faermann','3/2',3)
    drawnow
    
%    %*** save meshes
%    if counter < 10
%        filename = ['testEstSlpResidual/','0',int2str(counter),'.mat'];
%    else
%        filename = ['testEstSlpResidual/',int2str(counter),'.mat'];
%    end
%    eval(['save ',filename,' coordinates elements']);
    
    %*** stop if computation becomes unstable
    if nE(end) > 1000 && rho(end) > rho(end-5)
        counter
        break
    end
   
    
    %*** otherwise, adaptive rho-based mesh-refinement
    %marked = markElements(theta,rho_+osc_);
    %marked = markElements(theta,faermann_+osc_);
    marked = markElements(theta,1/4,mu_tilde_+osc_);
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements,marked);
    [coordinates,elements] = permuteMesh(coordinates,elements);
    
    counter = counter + 1;
end
