function [ fails, passes, rel_err, abs_err, EV, EK ] = testEvaluateVK( varargin )

% testEvaluateVK tests the functions evaluateV and evaluateK. 

    theta = 0.5;
    nEmax = 2000;
    number_of_tests = 100;

    if (nargin >= 2)
      error(sprintf(['Too many arguments in testEvaluateVK.\n' ...
                     '  Usage: testEvaluateVK([nEmax]);\n']));
    elseif (nargin >= 1)
      nEmax = varargin{1};
    end
    
    addpath evaluateVK
    addpath ..
    addpath ../lib
    
    coordinates = load('coordinates.dat');
    elements = load('elements.dat');
    
    while (size(elements,1) < nEmax)
        [coordinates_fine,elements_fine,father2son] ...
            = refineBoundaryMesh(coordinates,elements);
        [osc_fine, uDh_fine] ...
            = computeOscDirichlet(coordinates_fine,elements_fine,@g);
        V_fine = buildV(coordinates_fine,elements_fine);
        b_fine = buildSymmRHS(coordinates_fine,elements_fine,uDh_fine);
        x_fine = V_fine\b_fine;

        %*** compute (h-h/2)-error estimator tilde-mu
        mu_tilde_ = computeEstSlpMuTilde(coordinates,elements,father2son,x_fine);

        %*** compute data oscillations
        osc_fine = computeOscDirichlet(coordinates_fine,elements_fine,@g);
        osc_ = osc_fine(father2son(:,1))+osc_fine(father2son(:,2));

        %*** mark elements for refinement
        marked = markElements(theta,1/4,mu_tilde_+osc_);

        %*** generate new mesh
        [coordinates,elements] ...
            = refineBoundaryMesh(coordinates,elements,marked);

        %*** free obsolete memory
        clear V b
        clear coordinates_fine elements_fine father2son
        clear V_fine b_fine x_fine x
    end

    [osc, uDh] = computeOscDirichlet(coordinates, elements, @g);
    V = buildV(coordinates, elements);
    b = buildSymmRHS(coordinates, elements, uDh);
    x = V \ b;
    
    number_of_tests = floor(sqrt(number_of_tests))^2;
    sqrt_num = sqrt(number_of_tests);

    % Create nodes within domain
    for i=1:sqrt_num
      for j=1:sqrt_num
        z((i-1)*sqrt_num+j, 1) = i*0.5/(sqrt_num+1);
        z((i-1)*sqrt_num+j, 2) = j*0.5/(sqrt_num+1);
      end
    end

    % Create nodes on boundary
    zgamma = linspace(0.01, 0.49, 5);
    c = linspace(1, 1, 5);
    zgamma = [zgamma',0*c'; .5*c' zgamma'; zgamma',.5*c';0*c', zgamma'];

    z = [z; zgamma];

    number_of_tests = size(z,1);

    EV = evaluateV(coordinates, elements, x, z);
    EK = evaluateK(coordinates, elements, uDh, z);

    % Determine position of the first sol-value that corresponds to a
    % boundary node
    p = size(z,1) - size(zgamma,1) + 1;

    sol = EV - EK;

    % Modify sol on the boundary
    sol(p:end) = sol(p:end) + 0.5*g(z(p:end,:));

    ref = sinh(2*pi*z(:,1)).*cos(2*pi*z(:,2));
    abs_err = abs(ref-sol);
    rel_err = abs_err ./ abs(ref);

%    failed_tests = ( (rel_err > 1e-3) & (ref > 1e-12) );
     failed_tests =  ( (ref < 1e-12) & (abs_err > 1e-7) );

    for i=1:number_of_tests
      if (failed_tests(i))
        disp(...
          sprintf(['Test %g failed. z = [%g %g],\n' ...
                 '  abs_err = %g, rel_err = %g\n', ...
                 '  solution = %g, reference = %g\n'], ...
                  i, z(i,1), z(i,2), abs_err(i), rel_err(i), sol(i), ref(i)));

      end
    end

    fails = sum(failed_tests);
    passes = number_of_tests - fails;
