function testEvaluateW()

% Test HILBERT function evaluateW
% There holds W1 == 0
% Therefore 0=<g,W1>=<Wg,1> for arbitrary g
%
% i.e., integral mean of <1,Wg> has to be zero
% We compute <1,Wg> as follows:
%   <1,Wg> = sum( <1,Wg>_E ) and
%   <1,Wg>_E = |E|/2 * sum_j w_j * Wg(x_j)
%      with Gauss-points x_j and weights w_j
%
% Author: TF (24.05.2012)

addpath('../lib/')

%% TEST on \Omega = [0 1]^2
% g = x+y
g = @(x) sum(x,2);
coordinates = [0 0; 1 0; 1 1; 0 1];
elements = [1 2; 2 3; 3 4; 4 1];
% 

for i=1:8
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    %*** Nodal interpolation of g
    gh = g(coordinates);
    disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x+y\n',...
        computeIntWg1(coordinates,elements,gh),size(elements,1)))
end

%% TEST on \Omega = [0 1]^2
% g = x^2+y^2
g = @(x) sum(x.^2,2);
coordinates = [0 0; 1 0; 1 1; 0 1];
elements = [1 2; 2 3; 3 4; 4 1];
% 

for i=1:8
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    %*** Nodal interpolation of g
    gh = g(coordinates);
    disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x^2+y^2\n',...
        computeIntWg1(coordinates,elements,gh),size(elements,1)))
end

%% TEST on \Omega = [0 1]^2
% g = x+y
g = @(x) sum(x,2);
coordinates = [-1,-1;0,-1;1,-1;-1,0;0,0;1,0;-1,1;0,1;];
elements = [1,2;2,3;3,6;6,5;5,8;8,7;7,4;4,1;];
% 

for i=1:8
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    %*** Nodal interpolation of g
    gh = g(coordinates);
    disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x+y\n',...
        computeIntWg1(coordinates,elements,gh),size(elements,1)))
end

%% TEST on \Omega = L-shape
% g = x^2+y^2
g = @(x) sum(x.^2,2);
coordinates = [-1,-1;0,-1;1,-1;-1,0;0,0;1,0;-1,1;0,1;];
elements = [1,2;2,3;3,6;6,5;5,8;8,7;7,4;4,1;];
% 

for i=1:8
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    %*** Nodal interpolation of g
    gh = g(coordinates);
    disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x^2+y^2\n',...
        computeIntWg1(coordinates,elements,gh),size(elements,1)))
end


%% TEST on L-shape (adaptively refined mesh)
load('evaluateW/meshLshapeAdap.mat')
g = @(x) sum(x,2);
%*** Nodal interpolation of g
gh = g(coordinates);
disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x+y\n',...
    computeIntWg1(coordinates,elements,gh),size(elements,1)))
for i=1:1
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    %*** Nodal interpolation of g
    gh = g(coordinates);
    disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x+y\n',...
        computeIntWg1(coordinates,elements,gh),size(elements,1)))
end

load('evaluateW/meshLshapeAdap.mat')
g = @(x) sum(x.^2,2);
%*** Nodal interpolation of g
gh = g(coordinates);
disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x^2+y^2\n',...
    computeIntWg1(coordinates,elements,gh),size(elements,1)))
for i=1:1
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    %*** Nodal interpolation of g
    gh = g(coordinates);
    disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x+y\n',...
        computeIntWg1(coordinates,elements,gh),size(elements,1)))
end

load('evaluateW/meshLshapeAdap.mat')
% g S^1 function with random nodal values
gh = rand(size(coordinates,1),1);
disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = x+y\n',...
    computeIntWg1(coordinates,elements,gh),size(elements,1)))
for i=1:1
    [coordinates,elements] = refineBoundaryMesh(coordinates,elements);
    %*** Nodal interpolation of g
    gh = rand(size(coordinates,1),1);
    disp(sprintf('<1,Wg> = %d, nE = %i, g(x,y) = rand\n',...
        computeIntWg1(coordinates,elements,gh),size(elements,1)))
end


end


function intW = computeIntWg1(coordinates,elements,gh)

nE = size(elements,1);

%%%%%%%%%%%%%%%%%%%%%%%%%
%% ETA
eta = 0.5;
%%%%%%%%%%%%%%%%%%%%%%%%%

%*** 16-point rule on line segment
q_points = [-0.9894009349916499325961542, ...
    -0.9445750230732325760779884, ...
    -0.8656312023878317438804679, ...
    -0.7554044083550030338951012, ...
    -0.6178762444026437484466718, ...
    -0.4580167776572273863424194, ...
    -0.2816035507792589132304605, ...
    -0.0950125098376374401853193, ...
    0.0950125098376374401853193, ...
    0.2816035507792589132304605, ...
    0.4580167776572273863424194, ...
    0.6178762444026437484466718, ...
    0.7554044083550030338951012, ...
    0.8656312023878317438804679, ...
    0.9445750230732325760779884, ...
    0.9894009349916499325961542];

q_weights = [0.0271524594117540948517806; ...
    0.0622535239386478928628438; ...
    0.0951585116824927848099251; ...
    0.1246289712555338720524763; ...
    0.1495959888165767320815017; ...
    0.1691565193950025381893121; ...
    0.1826034150449235888667637; ...
    0.1894506104550684962853967; ...
    0.1894506104550684962853967; ...
    0.1826034150449235888667637; ...
    0.1691565193950025381893121; ...
    0.1495959888165767320815017; ...
    0.1246289712555338720524763; ...
    0.0951585116824927848099251; ...
    0.0622535239386478928628438; ...
    0.0271524594117540948517806];

gauss_order = length(q_weights);

%*** Element size h and normal vector nx
a = coordinates(elements(:,1),:);
b = coordinates(elements(:,2),:);
h = sqrt(sum( (b-a).^2,2));
nx = [b(:,2)-a(:,2) a(:,1)-b(:,1)]./[h h];

%*** build vector of all quadrature nodes as (nQ*nE x 2)-matrix
sx = reshape(a,2*nE,1)*(1-q_points) ...
    + reshape(b,2*nE,1)*(1+q_points);
sx = 0.5*reshape(sx',gauss_order*nE,2);

%*** sx2element(j) returns the element number k such that sx(j,:) lies on Ek
sx2element = reshape(repmat((1:nE),gauss_order,1),gauss_order*nE,1);

%*** evaluate W at quadrature nodes
wgsx = evaluateW(coordinates,elements,gh,sx,nx(sx2element,:),eta);

intW = h'*(0.5*reshape(wgsx,gauss_order,nE)'*q_weights);
end