function [fails, passes] = testFormulaIntegralI()

addpath ../work/lib
e = zeros(2,1);

n1 = zeros(2,1); n2 = zeros(2,1); n3 = zeros(2,1);

while (det([n2-n1,n3-n1]) < 1e-2)
  n1 = rand(2,1); n2 = rand(2,1); n3 = rand(2,1);
end

for i = 1:10
  for offset = [1.5 1.5 -1.5 -1.5; 1.5 -1.5 1.5 -1.5]
    x = rand(2,1) + offset;
    xi=rand(1,1);
    e = e + theTest(x,n1,n2,n3);
  end
end

fails = e(1);
passes = e(2);


function e = theTest(x,n1,n2,n3)
e = [0;0];

f = @(xi)(integrand(x,n1,n2,n3,xi));
   
ref = quad(f,0,1,1e-10);
ftest = formulaTest(x,n1,n2,n3); % MATLAB-Implementierung

fprintf('Reference value against formula test value: %g\n',abs(ref-ftest));

abs_err = abs(ref-ftest);
rel_err = abs_err / abs(ref);
if (rel_err > 1e-8)
    fprintf(['Failed: n1 = [%g,%g], n2 = [%g,%g], n3 = [%g,%g]\n'...
             '        x = [%g,%g], abs_err = %g, rel_err = %g\n', ...
             '        reference value = %g, test value = %g\n'],...
              n1(1),n1(2),n2(1),n2(2),n3(1),n3(2),...
              x(1),x(2),abs_err,rel_err, ref, ftest);
    e(1) = 1;
else
    e(2) = 1;
end

function val = integrand(x,n1,n2,n3,xi)
n1 = reshape(n1,2,1);
n2 = reshape(n2,2,1);
n3 = reshape(n3,2,1);
x = reshape(x,2,1);

u = n1-n2;
v = n1-n3;
w = x-n1;

for i=1:length(xi)
    a = u'*u;
    b = 2.*u'*(w+xi(i).*v);
    c = (w+xi(i).*v)'*(w+xi(i).*v);
    val(i) = (1-xi(i))*log(c+(1-xi(i))*b+(1-xi(i))^2*a);
end


function val = formulaTest(x,n1,n2,n3)
n1 = reshape(n1,2,1);
n2 = reshape(n2,2,1);
n3 = reshape(n3,2,1);
x = reshape(x,2,1);

u = n1-n2;
v = n1-n3;
w = x-n1;

val = 0.25 * (mxSlp(0, (v-u)./2, w+u+(v-u)./2) ...
              - mxSlp(1, (v-u)./2, w+u+(v-u)./2));


