function [fails, passes] = testFormulaIntegralII()

addpath ../work/lib
e = zeros(2,1);

n1 = zeros(2,1); n2 = zeros(2,1); n3 = zeros(2,1);

while (det([n2-n1,n3-n1]) < 1e-2)
  n1 = rand(2,1); n2 = rand(2,1); n3 = rand(2,1);
end

for i = 1:10
  for offset = [1.5 1.5 -1.5 -1.5; 1.5 -1.5 1.5 -1.5]
    x = rand(2,1) + offset;
    xi=rand(1,1);
    e = e + theTest(x,n1,n2,n3);
  end
end

fails = e(1);
passes = e(2);


function e = theTest(x,n1,n2,n3)
e = [0;0];

f = @(xi)(integrand(x,n1,n2,n3,xi));
fTilde = @(y)(transformedIntegrand(x,n1,n2,n3,y));

ref = quad(f,0,1,1e-10);
refTilde = quad(fTilde,-1,1,1e-10);
ftest = formulaTest(x,n1,n2,n3); % MATLAB-Implementierung der Stammfkt.

fprintf('Reference value against formula test value: %g\n',abs(ref-ftest));
fprintf('Reference value against substituted ref-value: %g\n\n',abs(ref-refTilde));

abs_err = abs(ref-ftest);
rel_err = abs_err / abs(ref);
if (rel_err > 1e-8)
    fprintf(['Failed: n1 = [%g,%g], n2 = [%g,%g], n3 = [%g,%g]\n'...
             '        x = [%g,%g], abs_err = %g, rel_err = %g\n', ...
             '        reference value = %g, test value = %g\n'],...
              n1(1),n1(2),n2(1),n2(2),n3(1),n3(2),...
              x(1),x(2),abs_err,rel_err, ref, ftest);
    e(1) = 1;
else
    e(2) = 1;
end

function val = integrand(x,n1,n2,n3,xi)
n1 = reshape(n1,2,1);
n2 = reshape(n2,2,1);
n3 = reshape(n3,2,1);
x = reshape(x,2,1);

u = n1-n2;
v = n1-n3;
w = x-n1;

%disp('|w|=');
%disp(norm(w));

for i=1:length(xi)
    a = u'*u;
    b = 2.*u'*(w+xi(i).*v);
    c = (w+xi(i).*v)'*(w+xi(i).*v);
    val(i) = b*log(c+(1-xi(i))*b+(1-xi(i))^2*a);
    
end

function val = transformedIntegrand(x,n1,n2,n3,y)
n1 = reshape(n1,2,1);
n2 = reshape(n2,2,1);
n3 = reshape(n3,2,1);
x = reshape(x,2,1);

u = n1-n2;
v = n1-n3;
w = x-n1;

for i=1:length(y)
    xi = (y(i)+1)/2;
    a = u'*u;
    b = 2.*u'*(w+xi.*v);
    c = (w+xi.*v)'*(w+xi.*v);
    
    %TESTEN DER SUBSTITUTION
%    val(i) = u'*(w+((y(i)+1)/2).*v) * ...
%        log(norm(((y(i)+1)/2).*(v-u)+(w+u),2)^2);

%    val(i) = (u'*w)*log(norm(y(i).*(v-u)./2+w+u+(v-u)./2,2)^2)+...
%             1/2*(y(i)+1)*(u'*v)*log(norm((v-u)./2.*y(i)+w+u+(v-u)./2,2)^2);

    val(i) = (u'*w)*log(norm(y(i).*(v-u)./2+w+u+(v-u)./2,2)^2)+...
             1/2*(u'*v)*log(norm((v-u)./2.*y(i)+w+u+(v-u)./2,2)^2)+...
             1/2*(u'*v)*y(i)*log(norm((v-u)./2.*y(i)+w+u+(v-u)./2,2)^2);
end

function val = formulaTest(x,n1,n2,n3)
n1 = reshape(n1,2,1);
n2 = reshape(n2,2,1);
n3 = reshape(n3,2,1);
x = reshape(x,2,1);

u = n1-n2;
v = n1-n3;
w = x-n1;

%TEST SLP VALUES TO CHECK WETHER ERROR STEMS FROM FORMULA OR INSTABILITY
testval1 = mxSlp(0, (v-u)./2,w+u+(v-u)./2);
f = @(s) integrandSlp(s,0,(v-u)./2,w+u+(v-u)./2);
reference1 = quad(f, -1,1, 1e2*eps);
if (abs(testval1-reference1)/abs(reference1)>1e-10)
    fprintf('mxSlp=%g, reference=%g\n',testval1,reference1);
    fprintf('k=0, v-u=[%g,%g], w+u+(v-u)/2=[%g,%g]\n',...
        v(1)-u(1),v(2)-u(2),w(1)+u(1)+(v(1)-u(1))/2,...
        w(2)+u(2)+(v(2)-u(2))/2);
    error('SLP INSTABILITY DETECTED!');
end

testval2 = mxSlp(1, (v-u)./2, w+u+(v-u)./2);
f = @(s) integrandSlp(s,1,(v-u)./2,w+u+(v-u)./2);
reference2 = quad(f, -1,1, 1e2*eps);
if (abs(testval2-reference2)/abs(reference2)>1e-10)
    fprintf('mxSlp=%g, reference=%g\n',testval2,reference2);
    fprintf('k=1, v-u=[%g,%g], w+u+(v-u)/2=[%g,%g]\n',...
        v(1)-u(1),v(2)-u(2),w(1)+u(1)+(v(1)-u(1))/2,...
        w(2)+u(2)+(v(2)-u(2))/2);
    error('SLP INSTABILITY DETECTED!');
end

fprintf('\n*** Formula test\n');
fprintf('First factor = %g\n',(2*(w'*u)+(u'*v)));
fprintf('Second factor = %g\n',(u'*v));
fprintf('***\n\n');

%val = 0.5 * (2*(w'*u)+(u'*v))*(mxSlp(0, (v-u)./2, w+u+(v-u)./2) ...
%- (u'*v)*mxSlp(1, (v-u)./2, w+u+(v-u)./2));
val = (u'*w)*mxSlp(0,(v-u)./2,w+u+(v-u)./2)+...
      1/2*(u'*v)*mxSlp(0,(v-u)./2,w+u+(v-u)./2)+...
      1/2*(u'*v)*mxSlp(1,(v-u)./2,w+u+(v-u)./2);

         
function val=integrandSlp(s,k,u,v)
val=zeros(size(s));
for ell=1:length(s)
   val(ell)=s(ell)^k*log(norm(u.*s(ell)+v,2)^2);
end
