function testOscNeumann
addpath('..');

%--------------------------------------------------------------------------

coordinates = [0 0; 1 0];
elements = [1 2];

%*** 1 Element, konstantes phi
disp('*** 1 Element, konstantes phi')
osc = computeOscNeumann(coordinates,elements,@phi0);
osc %*** muss 0 sein, da phi0 konstant
oscRef = computeOscNeumannRef(coordinates,elements,@phi0);
osc - oscRef

%*** 1 Element, affines phi
disp('*** 1 Element, affines phi')
osc = computeOscNeumann(coordinates,elements,@phi1);
osc-1/12 %*** muss 0 sein, da phi1 affin
oscRef = computeOscNeumannRef(coordinates,elements,@phi1);
osc - oscRef

%--------------------------------------------------------------------------

%*** zwei nicht-uniforme Elte, affines phi
disp('*** zwei nicht-uniforme Elte, affines phi')
coordinates = [0 0;1/5 0; 1 0];
elements = [1 2;2 3];
osc = computeOscNeumann(coordinates,elements,@phi1);
osc - [1/7500 ; 8/15*64/1000]
oscRef = computeOscNeumannRef(coordinates,elements,@phi1);
sum(osc-oscRef)

%--------------------------------------------------------------------------

%*** quadratisches phi, uniforme Netzverfeinerung
disp('*** quadratisches phi, uniforme Netzverfeinerung')

coordinates = [0 0; 1 0];
elements = [1 2];

steps = 18;
error = zeros(steps,1);
h = zeros(steps,1);
for j = 1:steps
    osc1 = computeOscNeumann(coordinates,elements,@phi2);
    osc2 = computeOscNeumannRef(coordinates,elements,@phi2);
    error(j) = sqrt(abs(sum(osc1-osc2)));
    h(j) = norm(coordinates(elements(1,1),:)-coordinates(elements(1,2),:));
    [coordinates,elements] = refineDPr(coordinates,elements);
    loglog(1./h(1:j),error(1:j),'.-');hold on;
    loglog(1./h(1:j),h(1:j).^2,'x-r');   %*** predicted by theory
    loglog(1./h(1:j),h(1:j).^2.5,'x-g'); %*** observed in practice
    legend('error','O(h^2)','O(h^{2.5})');
    drawnow
end

%--------------------------------------------------------------------------
return

%*** 1 Element, quadratisches phi (benoetigt gauss3!!)
disp('*** 1 Element, quadratisches phi')
osc = computeOscNeumann(coordinates,elements,@phi2);
osc - (1/5 - 1/9) %*** muss 0 sein ab gauss(3)
oscRef = computeOscNeumannRef(coordinates,elements,@phi2);
osc - oscRef

%*** zwei nicht-uniforme Elte, quadratisches phi (benoetigt gauss3!!)
disp('*** zwei nicht-uniforme Elte, quadratisches phi')
osc = computeOscNeumann(coordinates,elements,@phi2);
phi2_ = @(x) x.^2
phiT1 = 5*quad(phi2_,0,1/5);
phiT2 = 5/4*quad(phi2_,1/5,1);
oscT1 = 1/5*quad( @(x) (phi2_(x)-phiT1).^2 ,0,1/5);
oscT2 = 4/5*quad( @(x) (phi2_(x)-phiT2).^2 ,1/5,1);
sum(osc-[oscT1;oscT2])
oscRef = computeOscNeumannRef(coordinates,elements,@phi2);
sum(osc-oscRef)

%--------------------------------------------------------------------------
return

%*** zufaellige Elemente, affines phi
disp('*** zufaellige Elemente, affines phi')
[coordinates,elements] = refineDPr(coordinates,elements);
[coordinates,elements] = refineDPr(coordinates,elements);
for j = 1:10
    nE = size(elements,1);
    marked = find(rand(1,nE)>=.5);
    [coordinates,elements] = refineDPr(coordinates,elements,marked);
end
nE = size(elements,1)
osc = computeOscNeumann(coordinates,elements,@phi1);
oscRef = computeOscNeumannRef(coordinates,elements,@phi1);
sum(osc-oscRef)

%*** zufaellige Elemente, quadratisches phi
disp('*** zufaellige Elemente, quadratisches phi')
osc = computeOscNeumann(coordinates,elements,@phi2);
oscRef = computeOscNeumannRef(coordinates,elements,@phi2);
sum(osc-oscRef)

%==========================================================================

function y = phi0(x)
%*** phi(x,0) = const
y = 47*ones(size(x,1),1);

function y = phi1(x)
%*** phi(x,0) = x
y = x(:,1);

function y = phi2(x)
%*** phi(x,0) = x^2
y = x(:,1).^2;

%==========================================================================

function osc = computeOscNeumannRef(coordinates,elements,phi)

%*** gauss(2)
gaussNodes = [-1;1]/sqrt(3);
gaussWeights = [1 1];

%*** gauss(3)
gaussNodes = [-sqrt(3/5);0;sqrt(3/5)];
gaussWeights = [5 8 5]/9;

%*** gauss(5)
gaussNodes = [-sqrt(5+2*sqrt(10/7));-sqrt(5-2*sqrt(10/7));0;sqrt(5-2*sqrt(10/7));sqrt(5+2*sqrt(10/7))]/3;
gaussWeights = [322-13*sqrt(70),322+13*sqrt(70),128/225*900,322+13*sqrt(70),322-13*sqrt(70)]/900;

%*** the remaining code is independent of the chosen quadrature rule
nE = size(elements,1);
osc = zeros(nE,1);
for j = 1:nE
    a = coordinates(elements(j,1),:);
    b = coordinates(elements(j,2),:);
    h = norm(b-a);
    sx = ( (1-gaussNodes)*a + (1+gaussNodes)*b )/2;
    phisx = phi(sx);
    phiMean = 2\gaussWeights*phisx;
    oscsx = (phisx - phiMean).^2;
    osc(j) = 2\h^2*gaussWeights*oscsx;
end

%==========================================================================

function [coordinates,elements] = refineDPr(coordinates,elements,varargin)

nC = size(coordinates,1);
nE = size(elements,1);

if nargin == 2
    marked = 1:nE;
else
    marked = varargin{1};
end

%*** Build information on neighboring elements
elements4node = zeros(nC,2);
elements4node(elements(:,1),2) = (1:nE)';
elements4node(elements(:,2),1) = (1:nE)';
neighbors = [elements4node(elements(:,1),1) elements4node(elements(:,2),2)];

%*** Compute local mesh-size
a = coordinates(elements(:,1),:);
b = coordinates(elements(:,2),:);
h = sqrt( sum((a-b).^2,2) )';

%*** If Tj is marked + neighbour Tk satisfies hk >= kappa*hj,
%*** we mark Tk for refinement as well.
kappa = 2;

%*** Mark additional elements to keep \kappa(\TT) bounded
flag = zeros(1,nE);
flag(marked) = 1;
[tmp,idx] = sort(h);
for j = idx
    if flag(j)
       n = neighbors(j,:);
       n = n(find(n));
       n = n( find(h(n)>=kappa*h(j)) );
       flag(n) = 1;
    end
end
marked = find(flag);
nM = length(marked);

%*** Refine mesh
elements(nM+nE,1) = 0;
coordinates(nC+nM,1) = 0;
for j = 1:nM
  nr = marked(j);
  ab = elements(nr,:);
  % midpoint
  coordinates(nC+j,:) = 1/2*[1 1]*coordinates(ab,:);
  % new elements
  elements(nE+j,:) = [nC+j,ab(2)];
  elements(nr,:) = [ab(1),nC+j];
end 
