function fh = L2P1projection(vertices,triangles,f)

% vertices = (N x 3) matrix
% fh = (N x 3) matrix with nodal values for L2-projection of f onto P1

%*** quadrature rule on reference triangle Tref = conv{(0,0),(1,0),(0,1)}
tmp_pos = [6-sqrt(15) ; 9+2*sqrt(15) ; 6+sqrt(15) ; 9-2*sqrt(15) ; 7]/21;
quad_vertices = tmp_pos([1 1 ; 2 1 ; 1 2 ; 3 4 ; 3 3 ; 4 3 ; 5 5]);
tmp_wts = [155-sqrt(15) 155+sqrt(15) 270]/2400;
quad_weights = tmp_wts([1 1 1 2 2 2 3]);

%*** the remaining code is independent of the chosen quadrature rule
nT = size(triangles,1);
nQ = size(quad_vertices,1);

%*** first vertices of triangles and corresponding edge vectors
c1  = vertices(triangles(:,1),:);
d21 = vertices(triangles(:,2),:) - c1;
d31 = vertices(triangles(:,3),:) - c1;

%*** compute vector of triangle areas 2*area(T)
area2 = d21(:,1).*d31(:,2)-d21(:,2).*d31(:,1);

%*** build matrix of quadrature vertices by use of affine transformation of Tref
jacobian1 = reshape(repmat([d21(:,1);d31(:,1)],1,nQ)',nT*nQ,2);
jacobian2 = reshape(repmat([d21(:,2);d31(:,2)],1,nQ)',nT*nQ,2);
zref = repmat(quad_vertices,nT,1);
z = [ sum(zref.*jacobian1,2) sum(zref.*jacobian2,2) ] ...
    + reshape(repmat(c1(:),1,nQ)',nT*nQ,2);

%*** evaluate volume force f at all quadrature vertices z
fz = reshape(f(z),nQ,nT);

%*** form functions on Tref evaluated at all quadrature points
formfunction_zref = [ 1-quad_vertices(:,1)-quad_vertices(:,2), ...
                      quad_vertices(:,1), ...
                      quad_vertices(:,2) ];

%*** right-hand side vector for calculating the coefficients of the
%*** L2-orthogonal projection of F onto P1
rhs = ( repmat(quad_weights,3,1) .* formfunction_zref' ) * fz;

%*** inverse mass matrix times rhs yields coefficient vector fh
%*** stored in the form 3 x N, i.e., 3 nodal functions per triangle
fh = 6*[3,-1,-1;-1,3,-1;-1,-1,3] * rhs;

fh = fh';