function fT = projectP0Volume(nodes,triangles,f)

%*** quadrature rule on reference triangle Tref = conv{(0,0),(1,0),(0,1)}
tmp_pos = [6-sqrt(15) ; 9+2*sqrt(15) ; 6+sqrt(15) ; 9-2*sqrt(15) ; 7]/21;
quad_nodes = tmp_pos([1 1 ; 2 1 ; 1 2 ; 3 4 ; 3 3 ; 4 3 ; 5 5]);
tmp_wts = [155-sqrt(15) 155+sqrt(15) 270]/2400;
quad_weights = tmp_wts([1 1 1 2 2 2 3]);

%*** the remaining code is independent of the chosen quadrature rule
nT = size(triangles,1);
nQ = size(quad_nodes,1);

%*** first nodes of triangles and corresponding edge vectors
c1  = nodes(triangles(:,1),:);
d21 = nodes(triangles(:,2),:) - c1;
d31 = nodes(triangles(:,3),:) - c1;

%*** compute vector of triangle areas 2*area(T)
area2 = d21(:,1).*d31(:,2)-d21(:,2).*d31(:,1);

%*** build matrix of quadrature points
z = zeros(nQ*nT,2);
for j = 1:nT
    x = nodes(triangles(j,:),:);          % coordinates of nodes of Tj
    D = [x(2,:)-x(1,:) ; x(3,:)-x(1,:)];  % Jacobian of affine transformation
    z((j-1)*nQ+1:j*nQ,:) = quad_nodes*D + ones(nQ,1)*x(1,:); % z = phiT(r,s)   
end

%*** evaluate volume force f at all quadrature nodes z
fz = reshape(f(z),nQ,nT);

%*** compute integral mean
fT = (2*quad_weights*fz)';