function [fails, passes, relative_errors] = testPrecondV()
%testPrecondV: Performs several tests to verify the MEX-Function precondV
%              contained in precondV.c.
%
%Usage: [fails,passes,relative_errors]=testPrecondV()
%
%Description: TODO
%
%Output: fails ............. number of failed testcases.
%        passes ............ number of passed testcases.
%        relative_errors ... errors compaired to the reference solution.
%
%Authors: Markus Mayr
%
%(c) M. Aurada, M. Ebner, S. Ferraz-Leite,
%    P. Goldenits, M. Mayr, D. Praetorius

planed = 50;
relative_errors = zeros(planed,3);
passes = 0;
fails = 0;

for i=1:planed
  Vpre = rand(i*10);
  bpre = rand(i*10,1);
  for j=1:i*10
    Vpre(j,j) = 1;
  end

  dpre = rand(i*10,1);

  V = diag(dpre) * Vpre * diag(dpre);
  b = diag(dpre) * bpre;
  d = precondV(V,b);

  if (max(max(V-Vpre)) < 1e-15 && max(max(b-bpre)) < 1e-15 ...
        && max(max(1./d-dpre)) < 1e-15)
    passes = passes + 1;
  else
    fails = fails + 1;
  end

  relative_errors(i,1) = max(max(V-Vpre));
  relative_errors(i,2) = max(max(b-bpre));
  relative_errors(i,3) = max(max(1./d- dpre));

  clear V Vpre b bpre d dpre;
end

disp(sprintf('Totally passed %d out of %d tests. (%d fails)', ...
              passes, planed,fails));
