MAX_SIZE = 16000

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cd ../work
addpath lib

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

make realclean
make threaded-default-configure
make buildV
make buildW
make buildK

load examples/laplace/example01/coordinates.dat;
load examples/laplace/example01/elements.dat;

i = 1
[coordinates,elements]=refineBoundaryMesh(coordinates,elements);
while (size(coordinates,1) < MAX_SIZE)
  sThreaded(i) = size(elements, 1);
  tic; V=buildV(coordinates,elements); tVthreaded(i) = toc;
  clear V;
  tic; W=buildW(coordinates,elements); tWthreaded(i) = toc;
  clear W;
  tic; V=buildK(coordinates,elements); tKthreaded(i) = toc;
  clear K;
  i=i+1;
  [coordinates,elements]=refineBoundaryMesh(coordinates,elements);
end

clear coordinates elements

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear Configure
make realclean
make default-configure
make buildV
make buildW
make buildK

load examples/laplace/example01/coordinates.dat;
load examples/laplace/example01/elements.dat;

i = 1
[coordinates,elements]=refineBoundaryMesh(coordinates,elements);
while (size(coordinates,1) < MAX_SIZE)
  s(i) = size(elements, 1);
  tic; V=buildV(coordinates,elements); tV(i) = toc;
  clear V;
  tic; W=buildW(coordinates,elements); tW(i) = toc;
  clear W;
  tic; K=buildK(coordinates,elements); tK(i) = toc;
  clear K;
  i=i+1;
  [coordinates,elements]=refineBoundaryMesh(coordinates,elements);
end

clear coordinates elements

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

save('data.mat', 's', 'sThreaded', 'tV', 'tW', 'tK', ...
       'tVthreaded', 'tWthreaded', 'tKthreaded');

h = figure(1);
loglog(s, tV, 'r-', sThreaded, tVthreaded, 'b-', ...
     s, tW, 'g-', sThreaded, tWthreaded, 'm-', ...
     s, tK, 'y-', sThreaded, tKthreaded, 'k-');
legend('buildV', 'buildV parallelsiert', 'buildW', 'buildW paralleisiert', ...
       'buildK', 'buildK parallelsiert');
xlabel('Zahl der Elemente');
ylabel('Zeit in Sekunden');
title('Benötigte Zeit zum Aufbau der Matrizen');
saveas(h, 'time.fig');
print -depsc2 'time.eps';

h = figure(2);
plot(s, tV./tVthreaded, 'r-', ...
     s, tW./tWthreaded, 'g-', ...
     s, tK./tKthreaded, 'b-', ...
     s, feature('numCores') .* ones(1, i-1), 'b-.', ...
     s, ones(1, i-1), 'b--');
legend('buildV', 'buildW', 'buildK', 'optimaler Speed-up', 'Break-even point');
xlabel('Zahl der Randelemente');
ylabel('Geschwindigkeitszuwachs');
title('Geschwindigkeitszuwachs beim Aufbau der Matrizen');
saveas(h, 'speedup.fig');
print -depsc2 'speedup.eps';

rmpath lib