clear all;
close all;

addpath('../work/lib/');
addpath('../work/examples/FEMBEM/example01/');
load coordinates.dat
load elements.dat
load dirichlet.dat
elements(:,1)=[];
coordinates(:,1)=[];
dirichlet(:,1)=[];
errorH1=zeros(2,1);
errorNeumann=zeros(2,1);
errEst=zeros(2,4);
nE=zeros(2,1);
i=1;
while(nE(end)<=5000)
 
    nE(i)=size(elements,1);
    nC=size(coordinates,1);
    nD=size(dirichlet,1);
    %%% compute coarse solution
    %*** sort the boundary nodes last
    [coordinates,elements,dirichlet]=buildFemBemElements(coordinates,elements,dirichlet);
    [osc,U_0ell]=computeOscDirichlet(coordinates,dirichlet,@u0);
    %*** build the FEM-BEM ingridients
    [B,K,MMat,W]=buildSymmetricLHS(coordinates,elements,dirichlet);
    F=buildSymmetricRHS(coordinates,elements,dirichlet,@f,@phi_0,U_0ell(nC-nD+1:end),K,MMat,W);
    x=B\F;
    %*** compute H1 error of the solution
    errorH1(i)=sqrt(sum(computeErrH1(coordinates,elements,x(1:nC),@u1,@grad_u1)));
    %*** compute reliable error bound 
    errorNeumann(i)=sqrt(sum(computeErrNeumann(coordinates,dirichlet,x(nC+1:end),@phi)));
    %*** compute residual based error estimator
    [ind_volume,ind_boundary]=computeEstResidualSymmetric(coordinates,elements,dirichlet,x,@f,@phi_0,U_0ell);
    %*** store data
    errEst(i,1)=sqrt(sum(ind_volume));
    errEst(i,2)=sqrt(sum(ind_boundary));
    %%% compute fine solution
    %*** generate uniformly refined mesh
    [coordinates_fine,elements_fine,dirichlet_fine,father2vol,father2dir]=myRefineMesh(coordinates,elements,dirichlet);
    nC_fine=size(coordinates_fine,1);
    nD_fine=size(dirichlet_fine,1);  
    %*** sorting the nodes
    [coordinates_fine,elements_fine,dirichlet_fine]=buildFemBemElements(coordinates_fine,elements_fine,dirichlet_fine);
    [osc_fine,U_0ell_fine]=computeOscDirichlet(coordinates_fine,dirichlet_fine,@u0);
    [B,K,MMat,W]=buildSymmetricLHS(coordinates_fine,elements_fine,dirichlet_fine);
    F=buildSymmetricRHS(coordinates_fine,elements_fine,dirichlet_fine,@f,@phi_0,U_0ell_fine(nC_fine-nD_fine+1:end),K,MMat,W);
    x_fine=B\F;
    %*** compute the (h-h/2)-based error estimator
    [vol1,vol2,boundary]=computeEstMuFemBem(x_fine,coordinates_fine,elements_fine,...
        father2dir,coordinates,dirichlet);
    %*** store data
    errEst(i,3)=sqrt(sum(vol1+vol2));
    errEst(i,4)=sqrt(sum(boundary));
    %%% mark elements for refinement
    %*** in case of (h-h/2)-based refinement, we use the fine data
    %*** oscillations
    osc_proj=sum(osc_fine(father2dir),2);
    %*** mark elements for refinement
    [mark1,mark2]=markElements(0.25,sqrt(vol1+vol2),sqrt(boundary+osc_proj));
    %[mark1,mark2]=markElements(0.25,sqrt(ind_volume),sqrt(ind_boundary+osc));
    [coordinates,elements,dirichlet]=refineMesh(coordinates,elements,dirichlet,mark1,mark2);
    %*** plot mesh
    %figure();
    %trisurf(elements,coordinates(:,1),coordinates(:,2),ones(size(coordinates,1),1));
    
    %*** plot error bounds and error estimators
    %*** O(N^(-1/2)) and O(N^(-3/4))
    nO2=power(nE,-1/2);
    n03=power(nE,-3/4);
    loglog(nE,errorH1,'-xr',nE,errorNeumann,'-xb',nE,errEst(:,1),'-sg',nE,errEst(:,2),'-sm',...
         nE,errEst(:,3),'-*y',nE,errEst(:,4),'*-c',nE,nO2,'-k',nE,n03,'-k','LineWidth',2,'MarkerSize',10);
    legend('err(u)','err(phi)','eta(u)','eta(phi)','mu(u)',...
        'mu(phi)','Location','SouthWest');
    drawnow;
    %*** increment counter
    i=i+1;
end
