#!/usr/bin/perl

use warnings;
use strict;
use GraphViz;
use File::Find;
use File::Basename;

my @dirs = @ARGV;
die 'No base directory.' unless @dirs;

my @files;
find(\&wanted, @dirs);
my %nodes;
my %edges;
for my $file (@files) {
  my $fname = funcname($file);
  if (exists $nodes{$fname}) {
    $nodes{$fname} = "$fname (multiple files)";
  }
  else {
    $nodes{$fname} = $file;
  }
}

my $g = GraphViz->new();
$g->add_node($_, label => funcname($_)) for (values %nodes);

for my $file (@files) {
  my $rnode = $nodes{funcname($file)};
  open FH, '<', $file or die "Could not open matlab file: $!\n";
  while (my $line = <FH>) {
    for my $pattern (keys %nodes) {
      next if ($nodes{$pattern} eq $file);
      if ($line =~ /(.*)\s$pattern\(/ && $1 !~ /\%/) {
        unless (exists $edges{$rnode}{$nodes{$pattern}}) {
          $g->add_edge($rnode, $nodes{$pattern});
          $edges{$rnode}{$nodes{$pattern}} = 1;
        }
      }
    }
  }
  close FH or die "Could not close matlab file: $!\n";
}

print $g->as_canon;

sub wanted {
  my $f = $File::Find::name;
  if ($f !~ m/make/ && $f =~ m/(?:.*\/)*(.*?)\.m$/) { push @files, $f; }
}

sub funcname {
  my $file = shift || die;
  my $fname = basename($file);
  $fname =~ s/\.m$//;
  return $fname;
}
