#!/usr/bin/perl

use warnings;
use strict;

my $header = shift @ARGV;
if ($header eq '-h') {
  print <<EOF;
Verwendung:
  ./gen_mex_glue.pl headerfile.h return_type function_name parameter_types

Beispiele:
  ./gen_mex_glue.pl newtonPotential.h void computeN double*{p5,p5} \\
                                      double* double* int int double
    --> erstellt eine mexFunction für computeN.
  ./gen_mex_glue.pl test.h double*[p2,1] test double* int[p1,r]
    --> erstellt einen Stub für die Funktion double* test(double* a, int n)
        in test.h, wobei a ein Vektor der Länge n ist und der Rückgabevektor
        ebenfalls Länge n hat.
EOF
  exit 0;
}
elsif (@ARGV == 0) {
  @ARGV = split / /, $header;
  $header = shift @ARGV;
}
my $ret = shift @ARGV;
my $fct = shift @ARGV;
my @par = @ARGV;
my $str = join ' ', "$header:", $ret, ("$fct(" . join(", ", @par) . ")");
my $out = 0;
my $auto = 0;
my $in = 0;
my @bpar = qw();
my $i = 0;

die "Unknown return type"
  unless ($ret eq 'int' || $ret eq 'double' || $ret =~ m/double\*\{p?\d+.*?,p?\d+.*?\}/ || $ret eq 'void');
$out++ if ($ret ne 'void');

foreach my $p (@par) {
  if ($p =~ m/int/) {
    if ($p eq 'int') {
      push @bpar, { type => 'int', name => "i$in", num => $in, arg => 'in' };
      $in++;
    }
    elsif ($p =~ m/int\{p(\d+)\,(r|c)\}/) {
      push @bpar, { type => 'int', name => "a$auto", arg => 'auto',
                    sizeof => $1, rowcol => $2 };
      $auto++;
    }
    else {
      die;
    }
  }
  elsif ($p =~ m/double/) {
    if ($p eq 'double' || $p eq 'double*') {
      push @bpar, { type => $p, name => "i$in", num => $in, arg => 'in' };
      $in++;
    }
    elsif ($p =~ m/double\[(\d+)\]/) {
      push @bpar, { type => 'double*', name => "i$in", num => $in, arg => 'in',
                    len => $1 };
      $in++;
    }
    elsif ($p =~ m/double\*\{(p?\d+)(.*?)\,(p?\d+)(.*?)\}/) {
      push @bpar, { type => 'double*', name => "o$out", arg => 'out',
                    rows => $1, modrows => $2, cols => $3, modcols => $4};
      $out++;
    }
    else {
      die $p;
    }
  }
  else {
    die "Could not match parameter $p.\n";
  }
}

print <<EOF;
/* Auto-generated by tools/gen_mex_glue.pl */
/* (Call: $str) */

#include <stdlib.h>
#include <string.h>
#include "mex.h"
#include "$header"

void mexFunction(int nlhs, mxArray* plhs[], int nrhs, const mxArray* prhs[]) {
  const char* functionName = mexFunctionName();
  double* tmp = NULL;
  char errorMessage[255];
EOF

if ($ret ne 'void') {
  if ($ret =~ m/double\*/) {
    print "  double\* rval;\n";
  }
  else {
    print "  $ret rval;\n";
  }
}

foreach my $p (@bpar) {
  print "  $p->{type} $p->{name};\n";
}

print <<EOF;

  if (nlhs != $out || nrhs != $in) {
    sprintf(errorMessage,
      "Error in %s: Please use exactly $out output parameter and $in "
      "input parameters.", functionName);
    mexErrMsgTxt(errorMessage);
    return;
  }

EOF

foreach my $p (grep { $_->{arg} eq 'in' } @bpar) {
  if ($p->{type} eq 'int' || $p->{type} eq 'double') {
    print "  $p->{name} = mxGetScalar(prhs[$p->{num}]);\n";
  }
  else {
    print "  $p->{name} = mxGetPr(prhs[$p->{num}]);\n";
    if (exists $p->{len}) {
      print <<EOF;
  if (!((mxGetM(prhs[$p->{num}]) == $p->{len} && mxGetN(prhs[$p->{num}]) == 1)
      || mxGetN(prhs[$p->{num}]) == 1 && mxGetN(prhs[$p->{num}]) == $p->{len}))
  {
    sprintf(errorMessage,
      "Error in %s: Please provide an array with length $p->{len} as the\\n"
      "$p->{num} parameter.\\n", functionName);
    mexErrMsgTxt(errorMessage);
    return;
  }
EOF
    }
  }
}

foreach my $p (grep { $_->{arg} eq 'auto' } @bpar) {
  my $spar = $bpar[$p->{sizeof}-1] || die "Undefined parameter number $p->{sizeof}\n";
  my $varname = $spar->{name};
  ($spar->{type} eq 'double*' && $spar->{arg} eq 'in') || die "$varname is not a double*\n";
  if ($p->{rowcol} eq 'r') {
    print "  $p->{name} = mxGetM(prhs[$spar->{num}]);\n";
  }
  else {
    print "  $p->{name} = mxGetN(prhs[$spar->{num}]);\n";
  }
}

$i = 0;
if ($ret ne 'void') {
  $i++;
}

foreach my $p (grep { $_->{arg} eq 'out' } @bpar) {
  # type is double*
  my ($rows, $cols);
  if ($p->{rows} =~ m/p(\d+)/) {
    my $tp = $bpar[$1-1];
    if ($tp) {
      if ($tp->{type} eq 'double*' && $tp->{arg} eq 'in') {
        $rows = "mxGetM(prhs[$tp->{num}])";
      }
      elsif ($tp->{type} eq 'int') {
        $rows = $tp->{name};
      }
      else {
        die;
      }
    }
    else {
      die;
    }
  }
  else {
    $rows = $p->{rows};
  }
  if ($p->{cols} =~ m/p(\d+)/) {
    my $tp = $bpar[$1-1];
    if ($tp) {
      if ($tp->{type} eq 'double*' && $tp->{arg} eq 'in') {
        $cols = "mxGetN(prhs[$tp->{num}])";
      }
      elsif ($tp->{type} eq 'int') {
        $cols = $tp->{name};
      }
      else {
        die;
      }
    }
    else {
      die;
    }
  }
  else {
    $cols = $p->{cols};
  }
  print "  plhs[$i] = mxCreateDoubleMatrix($rows $p->{modrows}, $cols $p->{modcols}, mxREAL);\n";
  print "  $p->{name} = mxGetPr(plhs[$i]);\n";
}

if ($ret eq 'void') {
  print "  $fct(", (join ", ", map { $_->{name} } @bpar), ");\n";
}
else {
  print "  rval = $fct(", (join ", ", map { $_->{name} } @bpar), ");\n";
}

if ($ret =~ m/double\*\{(p?\d+)(.*?)\,(p?\d+)(.*?)\}/) {
  my $rowstr = $1; my $modrow = $2; my $colstr = $3; my $modcol = $4;
  my ($rows, $cols);
  if ($rowstr =~ m/p(\d+)/) {
    my $p = $bpar[$1-1];
    if ($p) {
      if ($p->{type} eq 'double*' && $p->{arg} eq 'in') {
        $rows = "mxGetM(prhs[$p->{num}])";
      }
      elsif ($p->{type} eq 'int') {
        $rows = $p->{name};
      }
      else {
        die;
      }
    }
    else {
      die;
    }
  }
  else {
    $rows = $rowstr;
  }
  if ($colstr =~ m/p(\d+)/) {
    my $p = $bpar[$1-1];
    if ($p) {
      if ($p->{type} eq 'double*' && $p->{arg} eq 'in') {
        $cols = "mxGetN(prhs[$p->{num}])";
      }
      elsif ($p->{type} eq 'int') {
        $cols = $p->{name};
      }
      else {
        die;
      }
    }
    else {
      die;
    }
  }
  else {
    $cols = $colstr;
  }
  print <<EOF;
  plhs[0] = mxCreateDoubleMatrix($rows $modrow, $cols $modcol, mxREAL);
  tmp = mxGetPr(plhs[0]);
  memcpy(tmp, rval, ($rows $modrow) * ($cols $modcol) * sizeof(double));
  free(rval);
EOF
}
elsif ($ret eq 'double' || $ret eq 'int') {
  print "  plhs[0] = mxCreateDoubleScalar((double) rval);\n";
}
elsif ($ret eq 'void') {}
else { die; }

print "}\n";

