#!/bin/sh

base_dir=$(pwd)
version=$1
target_dir=$base_dir/hilbert-$version/

read -p "I am creating a release from $base_dir in $target_dir. Ok? [y|n]" yn

case $yn in
  [Yy]* ) echo "Okay, creating release ..." ;;
  *     ) echo "Aborting."; exit ;;
esac

echo "Ensuring that documentation is up to date ..."
make -C docs/manual
cd docs/manual && ps2pdf dpr.ps && cd ../..

echo "Creating directory $target_dir ..."
mkdir -p $target_dir

echo "Copying source/, lib/, examples/ and make.m ..."
cp -r source/ $target_dir/
cp -r lib/ $target_dir/
cp -r examples/ $target_dir/
cp make.m $target_dir/

echo "Removing subversion data ..."
rm -rf $(find $target_dir -name "*.svn*")

echo "Removing Author and History information ..."
for dir in $(find $target_dir -type d); do tools/remove_author.pl $dir; done
tools/remove_author.pl $target_dir/make.m

echo "Copying documentation, license and README ..."
cp docs/manual/dpr.pdf $target_dir/documentation.pdf
cp license.pdf $target_dir/license.pdf
cp README $target_dir/README

echo "Creating Windows version of README file ..."
sed 's/$/
/' $target_dir/README > $target_dir/readme.txt

echo "Finished. Creating zip-Archive and tar archive ..."
zip -r hilbert-$version.zip hilbert-$version >/dev/null
tar cjf hilbert-$version.tar.bz2 hilbert-$version

echo "---"
echo "Archives hilbert-$version.zip and hilbert-$version.tar.bz2"
echo "are ready for shipping."

