#!/usr/bin/perl

=pod

=head1 NAME

remove_author.pl - Removes information about author and history from HILBERTs
                   source files.

=head1 USAGE

    remove_author.pl -h | --help
    remove_author.pl in [out]

If C<out> is not provided, it is set to C<in>. If C<in> is a file, it is copied
to C<out> until the first occurence of C<% Author:> or C</* AUTHOR:> or
C</*AUTHORS:> at the beginning of a line.

If C<in> is a directory, each C<*.m>, C<*.c> and C<*.h> file in that directory 
is copied to the directory C<out> until the first occurence of C<% Author:> or
C</* AUTHOR:> or C</* AUTHORS:> at the beginning of a line.

=head1 COPYRIGHT

Version 1.0,
(C) 2009 HILBERT-Team '09

=cut


use warnings;
use strict;

my $in = shift @ARGV;
my $out = shift @ARGV || $in;

unless ($in && $in ne '-h' && $in ne '--help') {
  print <<USAGE;
Usage:
    remove_author.pl -h | --help
    remove_author.pl in [out]

  If "out" is not provided, it is set to "in". If "in" is a file, it is copied
  to "out" until the first occurence of "% Author:" or "/* AUTHORS:" at the
  beginning of a line.

  If "in" is a directory, each *.m, *.c and *.h file in that directory is
  copied to the directory "out" until the first occurence of "% Author:" or
  "/* AUTHORS:" at the beginning of a line.
USAGE
  exit 0;
}

if (-d $in) {
  remove_author_in_dir($in, $out);
}
else {
  remove_author_in_file($in, $out);
}

sub remove_author_in_dir {
  my $in = shift
    or die "remove_author_in_dir requires one or two input arguments.\n";
  my $out = shift || $in;

  die "Output directory $out does not exist!\n" unless (-d $out);

  foreach my $infile (glob $in . '/*.[chm]') {
    $infile =~ m/$in\/?(.*)/;
    my $basename = $1;
    remove_author_in_file($infile, $out . '/' . $basename);
  }
}

sub remove_author_in_file {
  my $in = shift
    or die "remove_author_in_file requires one or two input arguments.\n";
  my $out = shift || $in;

  my $print_out = 1;

  open(IN, '<', $in) or die "Could not open input file $in: $!\n";
  my @lines = <IN>; # slurp all lines in.
  close IN;

  open(OUT, '>', $out) or die "Could not open output file $out: $!\n";
  select OUT;

  foreach my $line (@lines) {
    $print_out = 0 if ($line =~ /^\% Author\:.*/ 
                        || $line =~ /^\/\* AUTHORS?\:.*/);
    if ($print_out) {
      print $line;
    }
    else {
      if ($line !~ /^\%/ && $line !~ /^\/\*/ && $line ne "\n") {
        print STDERR <<EOF
> Warning: Deleting non-empty line from file $in (saved to $out):
>  $line
EOF
      }
    }
  }

  select STDOUT;
  close OUT;
}

